/*
* aes128_gcm.h
*
* 128 bit AES GCM mode
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
*
*    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


 
#ifndef _AES_GCM_H_
#define _AES_GCM_H_

#if defined(AESGCM_WITH_COMMON_KEY)
extern uint8_t common_expanded_aes_gcm_key[176];
#endif

typedef struct
{
#if !defined(AESGCM_WITH_COMMON_KEY)
    /*! Expanded AES key */
    uint8_t expanded_key[176];
#endif
    /*! The IV + counter buffer */
    uint8_t counter[AES_BLOCK_SIZE];
    /*! Initial IV + counter buffer encrypted with the AES key */
    uint8_t zero_encrypted[AES_BLOCK_SIZE];
    /*! "magic" hash key */
    uint8_t h[AES_BLOCK_SIZE];
    /*! Retained data buffer (AAD and data) */
    uint8_t y[AES_BLOCK_SIZE];
    /*! Block in progress */
    uint8_t r[AES_BLOCK_SIZE];
    /*! Length of block in progress */
    int r_len;
    /*! Track progress from initial state, through any AAD, to the data */
    int state;
    /*! The following should be 64 bit numbers, but this is overkill on a small machine. Even 32 bits is of dubious value. */
#if defined(AESGCM_WITH_64BIT_COUNTERS)
    /*! Length of the AAD, in bytes */
    uint64_t len_aad;
    /*! Length of the ciphertext, in bytes */
    uint64_t len_ct;
#else
    /*! Length of the AAD, in bytes */
    uint32_t len_aad;
    /*! Length of the ciphertext, in bytes */
    uint32_t len_ct;
#endif
} aes_gcm_state_t;

/* Initialise AES-GCM processing. The key is the 128 bit AES key. iv must be 12 bytes
    \param a A pointer to the aes_gcm_state_t structure which will be used for this AES-GCM processing
             context.
    \param iv The initialisation vector (salt)
    \param key The 16 byte AES encryption key. */
#if defined(AESGCM_WITH_COMMON_KEY)
void aes_gcm_init(aes_gcm_state_t *a, uint8_t iv[]);
#else
void aes_gcm_init(aes_gcm_state_t *a, uint8_t iv[], uint8_t key[]);
#endif

/* Add some AAD to the processing. If used, this must occur before any of the data */
int aes_gcm_add_aad(aes_gcm_state_t *a, uint8_t aad[], int aadlen, int encrypt);

/* Process some data. The returned value is the number of bytes added to the output buffer. */
int aes_gcm(aes_gcm_state_t *a, uint8_t out[], uint8_t data[], int datalen, int encrypt);

/* Finalise processing, and get the hash. The returned value is the number of bytes added to
   the output buffer. The hash is always 16 bytes. */
int aes_gcm_end(aes_gcm_state_t *a, uint8_t out[], uint8_t hash[], int encrypt);

#endif
