 /*
* aes128_gcm.c
*
* 128 bit AES GCM mode
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
*
*    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

 
#include <inttypes.h>
#include <string.h>

#include "aes128.h"
#include "aes128_gcm.h"

#if !defined(__inline__)
#define __inline__ inline
#endif

enum
{
    GCM_INITIAL = 0,
    GCM_IN_AAD = 1,
    GCM_AAD_FINISHED = 2
};

#if defined(AESGCM_WITH_COMMON_KEY)
extern uint8_t common_expanded_aes_gcm_key[176];
#endif

/* Use an assembly language implementation of the galois multiply, for speed. */
extern void gcm_galois_mult(uint8_t z[], const uint8_t x[]);

/* XOR a vector into another vector. */
static __inline__ void vec_xor(uint8_t z[], const uint8_t x[], int n)
{
    int i;

    for (i = 0;  i < n;  i++)
        z[i] ^= x[i];
}

#if defined(AESGCM_WITH_64BIT_COUNTERS)
static void len_to_bytes(uint64_t l, uint8_t b[])
{
    int i;

    /* The length needs converting from bytes to bits */
    l *= 8;
    for (i = 7;  i >= 0;  i--)
    {
        b[i] = l & 0xFF;
        l >>= 8;
    }
}
#else
static void len_to_bytes(uint32_t l, uint8_t b[])
{
    int i;

    /* The length needs converting from bytes to bits */
    l *= 8;
    for (i = 3;  i >= 0;  i--)
    {
        b[i] = 0;
        b[i + 4] = l & 0xFF;
        l >>= 8;
    }
}
#endif

static void ghash(uint8_t h[], uint8_t y[], uint8_t x[], int len)
{
    for (  ;  len > 0;  len -= AES_BLOCK_SIZE)
    {
        vec_xor(y, x, (len >= AES_BLOCK_SIZE)  ?  AES_BLOCK_SIZE  :  len);
        gcm_galois_mult(y, h);
        x += AES_BLOCK_SIZE;
    }
}

static void finalise_aad(aes_gcm_state_t *a)
{
    if (a->state == GCM_IN_AAD)
    {
        /* There is some AAD */
        if (a->r_len > 0)
        {
            /* There is some unprocessed AAD */
            ghash(a->h, a->y, a->r, a->r_len);
            a->len_aad += a->r_len;
            a->r_len = 0;
        }
    }
    a->state = GCM_AAD_FINISHED;
}

static int crunch_block(aes_gcm_state_t *a, uint8_t out[], int encrypt)
{
    int len;
    int i;

    if (a->r_len == 0)
        return 0;
    /* Increment the counter part (last 32 bits) of the IV + counter buffer */
    for (i = 15;  i >= 12;  i--)
    {
        if (++a->counter[i] != 0)
            break;
    }
    /* AES encrypt the IV + counter with the AES key */
    memcpy(out, a->counter, AES_BLOCK_SIZE);
#if defined(AESGCM_WITH_COMMON_KEY)
    aes128_encrypt(out, common_expanded_aes_gcm_key);
#else
    aes128_encrypt(out, a->expanded_key);
#endif
    /* XOR with the input data, which may be a partial block */
    memset(a->r + a->r_len, 0, AES_BLOCK_SIZE - a->r_len);
    memset(out + a->r_len, 0, AES_BLOCK_SIZE - a->r_len);
    vec_xor(out, a->r, a->r_len);
    len = a->r_len;
    a->len_ct += a->r_len;
    /* We mix the result of the "last" hash
       For encryption this may be AAD with ciphertext (out)
       For decryption this may be AAD with incoming ciphertext (a->r) */
    vec_xor(a->y, (encrypt)  ?  out  :  a->r, AES_BLOCK_SIZE);
    /* The residual buffer is now 'empty' */
    a->r_len = 0;
    /* Generate the next hash block */
    gcm_galois_mult(a->y, a->h);
    return len;
}

int aes_gcm_add_aad(aes_gcm_state_t *a, uint8_t aad[], int aadlen, int encrypt)
{
    int i;

    if (aadlen == 0)
        return 0;
    if (a->state >= GCM_AAD_FINISHED)
    {
        /* We have already started processing data blocks. All AAD must be handled before the data.
           Just ignore this. */
        return 0;
    }
    /* Mark that some AAD has been processed */
    a->state = GCM_IN_AAD;
    if ((aadlen + a->r_len) >= AES_BLOCK_SIZE)
    {
        /* We have one or more complete chunks of data */
        for (i = aadlen + a->r_len;  i >= AES_BLOCK_SIZE;  i -= AES_BLOCK_SIZE)
        {
            memcpy(a->r + a->r_len, aad, AES_BLOCK_SIZE - a->r_len);
            aad += (AES_BLOCK_SIZE - a->r_len);
            aadlen -= (AES_BLOCK_SIZE - a->r_len);
            ghash(a->h, a->y, a->r, AES_BLOCK_SIZE);
            a->len_aad = AES_BLOCK_SIZE;
            a->r_len = 0;
        }
    }
    if (aadlen)
    {
        /* We have a partial chunk to save for later. */
        memcpy(a->r + a->r_len, aad, aadlen);
        a->r_len += aadlen;
    }

    return 0;
}

int aes_gcm(aes_gcm_state_t *a, uint8_t out[], uint8_t data[], int datalen, int encrypt)
{
    int i;
    int len;

    if (datalen == 0)
        return 0;
    finalise_aad(a);
    len = 0;
    if ((datalen + a->r_len) >= AES_BLOCK_SIZE)
    {
        /* We have one or more complete chunks of data */
        for (i = datalen + a->r_len;  i >= AES_BLOCK_SIZE;  i -= AES_BLOCK_SIZE)
        {
            memcpy(a->r + a->r_len, data, AES_BLOCK_SIZE - a->r_len);
            data += (AES_BLOCK_SIZE - a->r_len);
            datalen -= (AES_BLOCK_SIZE - a->r_len);
            a->r_len = AES_BLOCK_SIZE;
            len += crunch_block(a, &out[len], encrypt);
        }
    }
    if (datalen)
    {
        /* We have a partial chunk to save for later. */
        memcpy(a->r + a->r_len, data, datalen);
        a->r_len += datalen;
    }

    return len;
}

int aes_gcm_end(aes_gcm_state_t *a, uint8_t out[], uint8_t hash[], int encrypt)
{
    int len;
    uint8_t buf[AES_BLOCK_SIZE];

    len = 0;
    if (a->r_len)
    {
        /* We have residual data to process. */
        finalise_aad(a);
        len = crunch_block(a, out, encrypt);
    }
    len_to_bytes(a->len_aad, buf);
    len_to_bytes(a->len_ct, buf + 8);
    ghash(a->h, a->y, buf, AES_BLOCK_SIZE);
    vec_xor(a->y, a->zero_encrypted, AES_BLOCK_SIZE);
    memcpy(hash, a->y, AES_BLOCK_SIZE);
    return len;
}

#if defined(AESGCM_WITH_COMMON_KEY)
void aes_gcm_init(aes_gcm_state_t *a, uint8_t iv[])
#else
void aes_gcm_init(aes_gcm_state_t *a, uint8_t iv[], uint8_t key[])
#endif
{
    memset(a, 0, sizeof(*a));
    /* The hash key is a block of zeros encrypted by the AES key. */
#if defined(AESGCM_WITH_COMMON_KEY)
    aes128_encrypt(a->h, common_expanded_aes_gcm_key);
#else
    aes128_expand_key(a->expanded_key, key);
    aes128_encrypt(a->h, a->expanded_key);
#endif

    /* Use the IV, which will always be 12 bytes in our application. There is a
       procedure for preparing an internal IV, when the supplied IV is not 12 bytes,
       but implementing this seems to be just overhead at present. */
    memcpy(a->counter, iv, 12);
    /* Start the counter field at 1 */
    a->counter[12] = 0;
    a->counter[13] = 0;
    a->counter[14] = 0;
    a->counter[15] = 1;
    memcpy(a->zero_encrypted, a->counter, AES_BLOCK_SIZE);
#if defined(AESGCM_WITH_COMMON_KEY)
    aes128_encrypt(a->zero_encrypted, common_expanded_aes_gcm_key);
#else
    aes128_encrypt(a->zero_encrypted, a->expanded_key);
#endif
}
