 /*
* aes128_wrap.c
*
* RFC3394 128 bit AES key wrapping
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
*
*    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

/* RFC3394 AES key wrapping */

#include <inttypes.h>
#include <stdlib.h>
#include <string.h>

#include "aes128.h"
#include "aes128_wrap.h"

int aes_wrap(uint8_t *cipher, const uint8_t *plain, int n, const uint8_t kek[176])
{
    uint8_t *a;
    uint8_t b[16];
    uint8_t *r;
    int i;
    int j;

    if ((n & 7) != 0)
        return -1;
    n >>= 3;
    a = cipher;
    r = cipher + 8;

    /* 1) Initialize variables. */
    memset(a, 0xA6, 8);
    memcpy(r, plain, 8*n);

    /* 2) Calculate intermediate values.
     * For j = 0 to 5
     *     For i=1 to n
     *         B = AES(K, A | R[i])
     *         A = MSB(64, B) ^ t where t = (n*j)+i
     *         R[i] = LSB(64, B)
     */
    for (j = 0;  j <= 5;  j++)
    {
        r = cipher + 8;
        for (i = 1;  i <= n;  i++)
        {
            memcpy(b, a, 8);
            memcpy(b + 8, r, 8);
            aes128_encrypt(b, kek);
            memcpy(a, b, 8);
            a[7] ^= n*j + i;
            memcpy(r, b + 8, 8);
            r += 8;
        }
    }

    /* 3) Output the results.
       The results are already in place. Just return their length. */
    return ((n + 1) << 3);
}

int aes_unwrap(uint8_t *plain, const uint8_t *cipher, int n, const uint8_t kek[176])
{
    uint8_t a[8];
    uint8_t b[16];
    uint8_t *r;
    int i;
    int j;

    if ((n & 7) != 0)
        return -1;
    /* 1) Initialize variables. */
    memcpy(a, cipher, 8);
    r = plain;
    memcpy(r, cipher + 8, n - 8);
    n = (n - 8) >> 3;

    /* 2) Compute intermediate values.
     * For j = 5 to 0
     *     For i = n to 1
     *         B = AES-1(K, (A ^ t) | R[i]) where t = n*j+i
     *         A = MSB(64, B)
     *         R[i] = LSB(64, B)
     */
    for (j = 5;  j >= 0;  j--)
    {
        r = plain + (n - 1)*8;
        for (i = n;  i >= 1;  i--)
        {
            memcpy(b, a, 8);
            b[7] ^= n*j + i;

            memcpy(b + 8, r, 8);
            aes128_decrypt(b, kek);
            memcpy(a, b, 8);
            memcpy(r, b + 8, 8);
            r -= 8;
        }
    }

    /* 3) Output results.
       The results are already in place. Test the IV is OK (this is the
       authentication step). If all is OK, return the length of the result. */
    for (i = 0;  i < 8;  i++)
    {
        if (a[i] != 0xA6)
            return -1;
    }

    return (n << 3);
}
