 /*
* aes128_gcm_tests.c
*
* This code tests the AES128-GCM (Galois Chain block cypher) code, using the test vectors
* provided by NIST. It can be run on a PC or in an MSP430 target. To build the code to run
* on a PC, define the symbol HOST_BUILD. To run the code on an MSP430 simply compile it,
* without any special action.

* The up to date version of the test vectors can always be downloaded from
* http://csrc.nist.gov/groups/STM/cavp/index.html
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
*
*    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


 
 
/* This code tests the AES128-GCM (Galois Chain block cypher) code, using the test vectors
   provided by NIST. It can be run on a PC or in an MSP430 target. To build the code to run
   on a PC, define the symbol HOST_BUILD. To run the code on an MSP430 simply compile it,
   without any special action.

   The up to date version of the test vectors can always be downloaded from
   http://csrc.nist.gov/groups/STM/cavp/index.html
*/

#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#if !defined(HOST_BUILD)
#include <msp430.h>
#endif

#include "aes128.h"
#include "aes128_gcm.h"

uint16_t SetVCore(uint8_t level);

#if !defined(HOST_BUILD)
static const char gcmDecrypt128[] =
{
#if defined(SMALL_TESTS)
#include "gcmDecrypt128_small.h"
#else
#include "gcmDecrypt128.h"
#endif
};

static const char gcmEncryptExtIV128[] =
{
#if defined(SMALL_TESTS)
#include "gcmEncryptExtIV128_small.h"
#else
#include "gcmEncryptExtIV128.h"
#endif
};
#endif

#if defined(AESGCM_WITH_COMMON_KEY)
uint8_t common_expanded_aes_gcm_key[176];
#endif

aes_gcm_state_t state;
char buf[132 + 1];
uint8_t key[2*AES_BLOCK_SIZE];
int keylen;
uint8_t iv[2*12];
int ivlen;
uint8_t pt[2*AES_BLOCK_SIZE];
int ptlen;
uint8_t aad[2*AES_BLOCK_SIZE];
int aadlen;
uint8_t ct[2*AES_BLOCK_SIZE];
int ctlen;
uint8_t tag[2*AES_BLOCK_SIZE];
int taglen;
int fail;
uint8_t hash[AES_BLOCK_SIZE];
uint8_t out[1024];
int outlen;
int encrypt;

int total_tests = 0;
int result = 0;

void done(int x)
{
#if defined(HOST_BUILD)
    exit(2);
#else
    result = x;
    for (;;)
        ;
#endif
}

void run_test(void)
{
    int i;

#if 0
    aes128_expand_key(common_expanded_aes_gcm_key, key);
    aes_gcm_init(&state, iv);
#else
    aes_gcm_init(&state, iv, key);
#endif
    outlen = 0;
    if (encrypt)
    {
        if (aadlen)
            aes_gcm_add_aad(&state, aad, aadlen, encrypt);
        if (ptlen)
            outlen += aes_gcm(&state, out, pt, ptlen, encrypt);
        outlen += aes_gcm_end(&state, out + outlen, hash, encrypt);
        for (i = 0;  i < taglen;  i++)
        {
            if (tag[i] != hash[i])
                break;
        }
        if (i < taglen)
        {
#if defined(HOST_BUILD)
            printf("Tag mismatch (%d)\n", fail);
            for (i = 0;  i < AES_BLOCK_SIZE;  i++)
                printf("%02X ", hash[i]);
            printf("\n");
            for (i = 0;  i < taglen;  i++)
                printf("%02X ", tag[i]);
            printf("\n");
#endif
            done(1);
        }
        for (i = 0;  i < ctlen;  i++)
        {
            if (ct[i] != out[i])
                break;
        }
        if (i < ctlen  ||  outlen != ctlen)
        {
#if defined(HOST_BUILD)
            printf("CT mismatch\n");
            printf("Len %d\n", ctlen);
            for (i = 0;  i < ctlen;  i++)
                printf("%02X ", ct[i]);
            printf("\n");
            printf("Len %d\n", outlen);
            for (i = 0;  i < outlen;  i++)
                printf("%02X ", out[i]);
            printf("\n");
#endif
            done(2);
        }
    }
    else
    {
        if (aadlen)
        	aes_gcm_add_aad(&state, aad, aadlen, encrypt);
        if (ptlen)
        	outlen += aes_gcm(&state, out, ct, ctlen, encrypt);
        outlen += aes_gcm_end(&state, out + outlen, hash, encrypt);
        for (i = 0;  i < taglen;  i++)
        {
            if (tag[i] != hash[i])
                break;
        }
        if ((!fail  &&  i < taglen)
            ||
            (fail  &&  i == taglen))
        {
#if defined(HOST_BUILD)
            printf("Tag mismatch (%d)\n", fail);
            for (i = 0;  i < AES_BLOCK_SIZE;  i++)
                printf("%02X ", hash[i]);
            printf("\n");
            for (i = 0;  i < taglen;  i++)
                printf("%02X ", tag[i]);
            printf("\n");
#endif
            done(3);
        }
        for (i = 0;  i < ptlen;  i++)
        {
            if (pt[i] != out[i])
                break;
        }
        if (!fail  &&  (i < ptlen  ||  outlen != ptlen))
        {
#if defined(HOST_BUILD)
            printf("PT mismatch\n");
            printf("Len %d\n", ptlen);
            for (i = 0;  i < ptlen;  i++)
                printf("%02X ", pt[i]);
            printf("\n");
            printf("Len %d\n", outlen);
            for (i = 0;  i < outlen;  i++)
                printf("%02X ", out[i]);
            printf("\n");
#endif
            done(4);
        }
    }
}

int hex_to_bin(int hex)
{
	int val;

	val = hex - '0';
	if (val > 9)
		val -= 7;
	if (val > 9)
		val -= 32;
	return val;
}

int extract(uint8_t buf[], char string[])
{
    char *s;
    uint8_t *t;

    /* Extract a string of binary octets from a strong of hex digits */
    s = string;
    t = buf;
    while (isxdigit(s[0])  &&  isxdigit(s[1]))
    {
        *t++ = (hex_to_bin(s[0]) << 4) | hex_to_bin(s[1]);
        s += 2;
    }
    return t - buf;
}

long int i;

int main(int argc, char *argv[])
{
#if 1
    long int j;
    static const char *s;
#endif

#if 0
#if !defined(HOST_BUILD)
    WDTCTL = WDTPW | WDTHOLD;
#endif

#if defined(__MSP430_HAS_UCS__)
    SetVCore(3);
    __bis_SR_register(SCG0);                        /* Disable the FLL control loop */
    UCSCTL6 = (UCSCTL6 | XT1DRIVE_3);               /* Highest drive setting for XT1 startup */
    while (SFRIFG1 & OFIFG)
    {
        /* Check OFIFG fault flag */
        UCSCTL7 &= ~(DCOFFG | XT1LFOFFG | XT2OFFG); /* Clear OSC fault flags */
        SFRIFG1 &= ~OFIFG;                          /* Clear OFIFG fault flag */
    }
    UCSCTL6 = (UCSCTL6 & ~(XT1DRIVE_3));            /* Reduce drive to something weaker */

    UCSCTL0 = 0;
    UCSCTL1 = DCORSEL_6;                            /* Set RSELx for DCO = 25MHz */
    UCSCTL2 = FLLD_2 | (192 - 1);                   /* Set DCO Multiplier for 25MHz */
                                                    /* Set FLL to 32768*4*192 => 25165824Hz */
    __bic_SR_register(SCG0);                        /* Enable the FLL control loop */

    UCSCTL5 |= DIVS__1;
    UCSCTL4 = SELM__DCOCLK | SELS__DCOCLK | SELA__XT1CLK;     /* 24MHz MCLK, 24MHz SMCLK, 32KHz ACLK */

    /* Suitable settings for a 6736 */
    PJDIR = BIT0 | BIT1 | BIT3;     /* SMCLK, MCLK, ACLK */
    PJSEL = BIT0 | BIT1 | BIT3;     /* SMCLK, MCLK, ACLK */
#endif

#if defined(__MSP430_HAS_UCS_RF__)
    __bis_SR_register(SCG0);                        /* Disable the FLL control loop */
    UCSCTL6 = (UCSCTL6 | XT1DRIVE_3);               /* Highest drive setting for XT1 startup */
#if 0
    while (SFRIFG1 & OFIFG)
    {
        /* Check OFIFG fault flag */
        UCSCTL7 &= ~(DCOFFG | XT1LFOFFG | XT2OFFG); /* Clear OSC fault flags */
        SFRIFG1 &= ~OFIFG;                          /* Clear OFIFG fault flag */
    }
#endif
    UCSCTL6 = (UCSCTL6 & ~(XT1DRIVE_3));            /* Reduce drive to something weaker */

    UCSCTL1 = DCORSEL_5;                            /* Set RSELx for DCO = 16 MHz */
    UCSCTL2 = FLLD_1 + 255;                         /* Set DCO Multiplier for 8MHz */
                                                    /* (N + 1) * FLLRef = Fdco */
                                                    /* (63 + 1) * 32768 = 2MHz */
                                                    /* Set FLL Div = fDCOCLK/2 */
    __bic_SR_register(SCG0);                        /* Enable the FLL control loop */

    UCSCTL5 |= DIVS__1;
    UCSCTL4 = SELM__DCOCLK | SELS__DCOCLK | SELA__XT1CLK;     //16MHz MCLK, 16MHz SMCLK, 32KHz ACLK

    /* Feed the clock out */
    P3SEL |= BIT7;
    P3DIR |= BIT7;
#endif

    P1DIR |= BIT0;
#endif

    keylen = 0;
    ivlen = 0;
    ptlen = 0;
    aadlen = 0;
    ctlen = 0;
    taglen = 0;
    fail = 0;

    total_tests = 0;
#if !defined(HOST_BUILD)
    encrypt = 0;
    s = gcmDecrypt128;
    for (i = 0;  ;  i++)
    {
        if (s[i] == '\0')
        {
            if (s == gcmEncryptExtIV128)
                break;
            encrypt = 1;
            s = gcmEncryptExtIV128;
            i = 0;
        }
        for (j = 0;  s[i + j];  j++)
        {
            buf[j] = s[i + j];
            if (buf[j] == '\n')
                break;
        }
        buf[j + 1] = '\0';
        i += j;
#define strncasecmp strncmp
#else
    encrypt = (argc > 1);
    while (fgets(buf, 132, stdin))
    {
#endif
        if (strncasecmp(buf, "Count ", 6) == 0)
        {
            /* Nothing to do */
#if defined(HOST_BUILD)
            printf("Count\n");
#endif
        }
        else if (strncasecmp(buf, "Key ", 4) == 0)
        {
            keylen = extract(key, buf + 6);
#if defined(HOST_BUILD)
            printf("Key %d\n", keylen);
#endif
        }
        else if (strncasecmp(buf, "IV ", 3) == 0)
        {
            ivlen = extract(iv, buf + 5);
#if defined(HOST_BUILD)
            printf("IV %d\n", ivlen);
#endif
        }
        else if (strncasecmp(buf, "PT ", 3) == 0)
        {
            ptlen = extract(pt, buf + 5);
#if defined(HOST_BUILD)
            printf("PT %d\n", ptlen);
#endif
        }
        else if (strncasecmp(buf, "AAD ", 4) == 0)
        {
            aadlen = extract(aad, buf + 6);
#if defined(HOST_BUILD)
            printf("AAD %d\n", aadlen);
#endif
        }
        else if (strncasecmp(buf, "CT ", 3) == 0)
        {
            ctlen = extract(ct, buf + 5);
#if defined(HOST_BUILD)
            printf("CT %d\n", ctlen);
#endif
        }
        else if (strncasecmp(buf, "Tag ", 4) == 0)
        {
            taglen = extract(tag, buf + 6);
#if defined(HOST_BUILD)
            printf("Tag %d\n", taglen);
#endif
        }
        else if (strncasecmp(buf, "[", 1) == 0)
        {
            /* Nothing to do */
#if defined(HOST_BUILD)
            printf("%s", buf);
#endif
        }
        else if (strncasecmp(buf, "FAIL", 4) == 0)
        {
            fail = 1;
#if defined(HOST_BUILD)
            printf("FAIL\n");
#endif
        }
        else if (buf[0] == ' '  ||  buf[0] == '\n'  ||  buf[0] == '\r')
        {
            run_test();

            keylen = 0;
            ivlen = 0;
            ptlen = 0;
            aadlen = 0;
            ctlen = 0;
            taglen = 0;
            fail = 0;

            total_tests++;
        }
        else
        {
            /* Something unexpected in the test data */
            //done(5);
        }
    }
#if defined(HOST_BUILD)
    printf("Total tests %d\n", total_tests);
#endif
    /* Successful completion */
    done(0);
    return 0;
}

#if defined(__TI_COMPILER_VERSION__)

#if defined(__MSP430AFE253__)
#pragma vector = PORT2_VECTOR, \
		 	 	 PORT1_VECTOR, \
		 	 	 TIMERA1_VECTOR, \
		 	 	 TIMERA0_VECTOR, \
	 	 	     /* USART0TX_VECTOR, */ \
 	 	 	 	 /* USART0RX_VECTOR, */ \
 	 	 	 	 /* WDT_VECTOR, */ \
 	 	 	 	 /* SD24_VECTOR, */ \
 	 	         NMI_VECTOR
#elif defined(__MSP430F427__)  ||  defined(__MSP430F427A__)
#pragma vector = /* BASICTIMER_VECTOR, */ \
		 	 	 PORT2_VECTOR, \
		 	 	 PORT1_VECTOR, \
		 	 	 TIMERA1_VECTOR, \
		 	 	 /* TIMERA0_VECTOR, */ \
	 	 	     /* USART0TX_VECTOR, */ \
 	 	 	 	 /* USART0RX_VECTOR, */ \
 	 	 	 	 WDT_VECTOR, \
 	 	 	 	 /* SD16_VECTOR, */ \
 	 	         NMI_VECTOR
#elif defined(__MSP430FE427__)  ||  defined(__MSP430FE427A__)  ||  defined(__MSP430FE4272__)
#pragma vector = /* BASICTIMER_VECTOR, */ \
		 	 	 PORT2_VECTOR, \
		 	 	 PORT1_VECTOR, \
		 	 	 TIMERA1_VECTOR, \
		 	 	 /* TIMERA0_VECTOR, */ \
	 	 	     /* USART0TX_VECTOR, */ \
 	 	 	 	 /* USART0RX_VECTOR, */ \
 	 	 	 	 WDT_VECTOR, \
 	 	 	 	 /* SD16_VECTOR, */ \
                 /* ESP430_VECTOR, */ \
 	 	         NMI_VECTOR
#elif defined(__MSP430F4794__)
#pragma vector = /* BASICTIMER_VECTOR, */ \
				 PORT2_VECTOR, \
				 USCIAB1TX_VECTOR, \
				 USCIAB1RX_VECTOR, \
				 PORT1_VECTOR, \
				 TIMERA1_VECTOR, \
				 TIMERA0_VECTOR, \
				 /* SD16A_VECTOR, */ \
				 /* USCIAB0TX_VECTOR, */ \
				 /* USCIAB0RX_VECTOR, */ \
				 WDT_VECTOR, \
				 COMPARATORA_VECTOR, \
				 TIMERB1_VECTOR, \
				 TIMERB0_VECTOR, \
				 NMI_VECTOR
#elif defined(__MSP430F47197__)  ||  defined(__MSP430F47187__)
#pragma vector = /* DMA_VECTOR, */ \
                 /* BASICTIMER_VECTOR, */ \
				 PORT2_VECTOR, \
				 USCIAB1TX_VECTOR, \
				 /* USCIAB1RX_VECTOR, */ \
				 PORT1_VECTOR, \
				 TIMERA1_VECTOR, \
				 TIMERA0_VECTOR, \
				 /* SD16A_VECTOR, */ \
				 /* USCIAB0TX_VECTOR, */ \
				 /* USCIAB0RX_VECTOR, */ \
				 WDT_VECTOR, \
				 COMPARATORA_VECTOR, \
				 TIMERB1_VECTOR, \
				 TIMERB0_VECTOR, \
				 NMI_VECTOR
#elif defined(__MSP430F6736__)
#pragma vector = /* RTC_VECTOR, */ \
	             LCD_C_VECTOR, \
                 TIMER3_A1_VECTOR, \
                 TIMER3_A0_VECTOR, \
                 PORT2_VECTOR, \
                 TIMER2_A1_VECTOR, \
                 TIMER2_A0_VECTOR, \
                 PORT1_VECTOR, \
                 TIMER1_A1_VECTOR, \
                 TIMER1_A0_VECTOR, \
                 DMA_VECTOR, \
                 AUX_VECTOR, \
                 USCI_A2_VECTOR, \
                 /* USCI_A1_VECTOR, */ \
                 TIMER0_A1_VECTOR, \
                 /* TIMER0_A0_VECTOR, */ \
                 /* SD24B_VECTOR, */ \
                 /* ADC10_VECTOR, */ \
                 USCI_B0_VECTOR, \
                 /* USCI_A0_VECTOR, */ \
                 /* WDT_VECTOR, */ \
                 UNMI_VECTOR, \
                 SYSNMI_VECTOR
#elif defined(__MSP430F6779__)
#pragma vector = AES_VECTOR, \
                 COMP_B_VECTOR, \
                 RTC_VECTOR, \
	             LCD_C_VECTOR, \
                 TIMER3_A1_VECTOR, \
                 TIMER3_A0_VECTOR, \
                 PORT2_VECTOR, \
                 TIMER2_A1_VECTOR, \
                 TIMER2_A0_VECTOR, \
                 PORT1_VECTOR, \
                 USCI_B1_VECTOR, \
                 USCI_A3_VECTOR, \
                 TIMER1_A1_VECTOR, \
                 TIMER1_A0_VECTOR, \
                 DMA_VECTOR, \
                 AUX_VECTOR, \
                 USCI_A2_VECTOR, \
                 USCI_A1_VECTOR, \
                 TIMER0_A1_VECTOR, \
                 TIMER0_A0_VECTOR, \
                 SD24B_VECTOR, \
                 ADC10_VECTOR, \
                 USCI_B0_VECTOR, \
                 USCI_A0_VECTOR, \
                 WDT_VECTOR, \
                 UNMI_VECTOR, \
                 SYSNMI_VECTOR
#elif defined(__MSP430F67791__)
#pragma vector = COMP_B_VECTOR, \
                 RTC_VECTOR, \
	             LCD_C_VECTOR, \
                 TIMER3_A1_VECTOR, \
                 TIMER3_A0_VECTOR, \
                 PORT2_VECTOR, \
                 TIMER2_A1_VECTOR, \
                 TIMER2_A0_VECTOR, \
                 PORT1_VECTOR, \
                 USCI_B1_VECTOR, \
                 USCI_A3_VECTOR, \
                 TIMER1_A1_VECTOR, \
                 TIMER1_A0_VECTOR, \
                 DMA_VECTOR, \
                 AUX_VECTOR, \
                 USCI_A2_VECTOR, \
                 USCI_A1_VECTOR, \
                 TIMER0_A1_VECTOR, \
                 TIMER0_A0_VECTOR, \
                 SD24B_VECTOR, \
                 ADC10_VECTOR, \
                 USCI_B0_VECTOR, \
                 USCI_A0_VECTOR, \
                 WDT_VECTOR, \
                 UNMI_VECTOR, \
                 SYSNMI_VECTOR
#elif defined(__MSP430I4020__)
#pragma vector = PORT1_VECTOR, \
		         TIMER0_A1_VECTOR, \
		         TIMER0_A0_VECTOR, \
                 USCI_B0_VECTOR, \
                 /* USCI_A0_VECTOR, */ \
                 /* WDT_VECTOR, */ \
                 VMON_VECTOR, \
                 /* SD24_VECTOR, */ \
                 NMI_VECTOR
#elif defined(__MSP430i2040__)  ||  defined(__MSP430i2041__)
#pragma vector = PORT1_VECTOR, \
		         PORT2_VECTOR, \
                 TIMER0_A1_VECTOR, \
                 TIMER0_A0_VECTOR, \
                 TIMER1_A1_VECTOR, \
                 TIMER1_A0_VECTOR, \
                 USCI_B0_VECTOR, \
                 /* USCI_A0_VECTOR, */ \
                 WDT_VECTOR, \
                 VMON_VECTOR, \
                 /* SD24_VECTOR, */ \
                 NMI_VECTOR
#else
#error Device not recognised
#endif
__interrupt void ISR_trap(void)
{
    /* The following will cause an access violation which results in a PUC reset */
    WDTCTL = 0;
}
#endif
