 /*
* aes128_tests2.c
*
* This code tests the AES128-CBC (Chain block cypher) code, using the test vectors
* from RFC3602. It can be run on a PC or in an MSP430 target. Testing the CBC mode
* also tests the underlyign AES128 code. To build the code to run on a PC, define the
* symbol HOST_BUILD. To run the code on an MSP430 simply compile it, without any special
* action. 
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
*
*    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


 
 
/* This code tests the AES128-CBC (Chain block cypher) code, using the test vectors
   from RFC3602. It can be run on a PC or in an MSP430 target. Testing the CBC mode
   also tests the underlyign AES128 code. To build the code to run on a PC, define the
   symbol HOST_BUILD. To run the code on an MSP430 simply compile it, without any special
   action. */

#include <inttypes.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#if !defined(HOST_BUILD)
#include <msp430.h>
#endif

#include "aes128.h"

uint8_t common_expanded_aes_gcm_key[176];

char buf[132 + 1];
uint8_t key[2*AES_BLOCK_SIZE];
int keylen;
uint8_t iv[2*12];
int ivlen;
uint8_t pt[2*AES_BLOCK_SIZE];
int ptlen;
uint8_t aad[2*AES_BLOCK_SIZE];
int aadlen;
uint8_t ct[2*AES_BLOCK_SIZE];
int ctlen;
uint8_t tag[2*AES_BLOCK_SIZE];
int taglen;
uint8_t plaintext[2*AES_BLOCK_SIZE];
int plaintextlen;
uint8_t ciphertext[2*AES_BLOCK_SIZE];
int ciphertextlen;
int fail;
int total_tests;
uint8_t hash[AES_BLOCK_SIZE];
uint8_t out[1024];
int outlen;
int encrypt;
int monte_carlo;

void log_buffer(const uint8_t buf[], int len)
{
    int i;

    for (i = 0;  i < len;  i++)
        printf(" %02x", buf[i]);
    printf("\n");
}

int extract(uint8_t buf[], char string[])
{
    char *s;
    uint8_t *t;
    int i;
    
    s = string;
    t = buf;
    while (isxdigit(s[0])  &&  isxdigit(s[1]))
    {
        sscanf(s, "%2x", &i);
        s += 2;
        *t++ = i;
    }
    return t - buf;
}

void failure(const char *msg)
{
#if defined(HOST_BUILD)
    fprintf(stderr, "%s\n", msg);
    exit(2);
#else
    for (;;)
        __no_operation();
#endif
}

void run_ebc_test(void)
{
    int i;

#if 1
    aes128_expand_key(common_expanded_aes_gcm_key, key);
#endif
    outlen = 0;
    if (plaintextlen  &&  ciphertextlen)
    {
        if (encrypt)
        {
            aes128_encrypt(plaintext, common_expanded_aes_gcm_key);
            if (memcmp(plaintext, ciphertext, 16))
            {
                failure("Error in encode\n");
                printf("%d:", plaintextlen);
                log_buffer(plaintext, plaintextlen);
                printf("%d:", ciphertextlen);
                log_buffer(ciphertext, ciphertextlen);
            }
        }
        else
        {
            aes128_decrypt(ciphertext, common_expanded_aes_gcm_key);
            if (memcmp(plaintext, ciphertext, 16))
            {
                failure("Error in decode\n");
                printf("%d:", plaintextlen);
                log_buffer(plaintext, plaintextlen);
                printf("%d:", ciphertextlen);
                log_buffer(ciphertext, ciphertextlen);
            }
        }
    }
}

void run_cbc_test(void)
{
    int i;

#if 1
    aes128_expand_key(common_expanded_aes_gcm_key, key);
#endif
    outlen = 0;
    if (plaintextlen  &&  ciphertextlen)
    {
        if (encrypt)
        {
            memcpy(buf, iv, ivlen);
            aes128_encrypt_cbc(plaintext, buf, common_expanded_aes_gcm_key);
            if (memcmp(plaintext, ciphertext, 16))
            {
                failure("Error in encode\n");
                printf("%d:", plaintextlen);
                log_buffer(plaintext, plaintextlen);
                printf("%d:", ciphertextlen);
                log_buffer(ciphertext, ciphertextlen);
            }
        }
        else
        {
            memcpy(buf, iv, ivlen);
            aes128_decrypt_cbc(ciphertext, buf, common_expanded_aes_gcm_key);
            if (memcmp(plaintext, ciphertext, 16))
            {
                failure("Error in decode\n");
                printf("%d:", plaintextlen);
                log_buffer(plaintext, plaintextlen);
                printf("%d:", ciphertextlen);
                log_buffer(ciphertext, ciphertextlen);
            }
        }
    }
}

/* XOR a vector into another vector. */
static __inline__ void vec_xor(uint8_t z[], const uint8_t x[], int n)
{
    int i;

    for (i = 0;  i < n;  i++)
        z[i] ^= x[i];
}

int ecb_monte_carlo_test(uint8_t initial_key[16], uint8_t initial_pt[16])
{
    int i;
    int j;
    uint8_t key[16];
    uint8_t pt[16];
    uint8_t ct[16];
    uint8_t ref[16];
    uint8_t ptx[16];

    memcpy(key, initial_key, 16);
    memcpy(pt, initial_pt, 16);

    /* AESAVS 6.4.1 */
    /* Follow the algorithm in AESAVS */
    printf("Monte Carlo ECB encrypt\n");
    for (i = 0;  i <= 99;  i++)
    {
        printf("KEY =");
        log_buffer(key, 16);
        printf("PLAINTEXT =");
        log_buffer(pt, 16);

        aes128_expand_key(common_expanded_aes_gcm_key, key);
        memcpy(ct, pt, 16);
        for (j = 0;  j <= 999;  j++)
            aes128_encrypt(ct, common_expanded_aes_gcm_key);
        printf("CIPHERTEXT =");
        log_buffer(ct, 16);
        printf("\n");
        memcpy(ptx, ct, 16);
        for (j = 0;  j <= 999;  j++)
            aes128_decrypt(ptx, common_expanded_aes_gcm_key);
        if (memcmp(pt, ptx, 16))
        {
            printf("Monte Carlo EBC key restore failure\n");
            printf("Error\n");
            exit(2);
        }
        memcpy(pt, ct, 16);
        vec_xor(key, ct, 16);
    }
    printf("Monte Carlo EBC test passed\n\n");
}

int cbc_monte_carlo_test(uint8_t initial_key[16], uint8_t initial_iv[16], uint8_t initial_pt[16])
{
    int i;
    int j;
    uint8_t key[16];
    uint8_t pt[16];
    uint8_t ct[16];
    uint8_t iv[16];
    uint8_t ref[16];
    uint8_t prev_enc[16];
    uint8_t prev_dec[16];
    uint8_t prev_ct[16];
    uint8_t prev_ct2[16];
    uint8_t prev_iv[16];
    uint8_t ctx[16];
    uint8_t ptx[16];

    memcpy(key, initial_key, 16);
    memcpy(iv, initial_iv, 16);
    memcpy(pt, initial_pt, 16);
    
    /* AESAVS 6.4.2 */
    printf("Monte Carlo CBC encrypt\n");
    for (i = 0;  i <= 99;  i++)
    {
        printf("KEY =");
        log_buffer(key, 16);
        printf("IV =");
        log_buffer(iv, 16);
        printf("PLAINTEXT =");
        log_buffer(pt, 16);

        aes128_expand_key(common_expanded_aes_gcm_key, key);
        memcpy(ct, pt, 16);
        memcpy(prev_enc, iv, 16);
        memcpy(prev_dec, iv, 16);
        aes128_encrypt_cbc(ct, prev_enc, common_expanded_aes_gcm_key);
        memcpy(ptx, ct, 16);
        aes128_decrypt_cbc(ptx, prev_dec, common_expanded_aes_gcm_key);
        memcpy(prev_ct, ct, 16);
        memcpy(ct, iv, 16);
        memcpy(prev_iv, iv, 16);
        for (j = 1;  j <= 999;  j++)
        {
            memcpy(ctx, ct, 16);
            aes128_encrypt_cbc(ct, prev_enc, common_expanded_aes_gcm_key);
            memcpy(ptx, ct, 16);
            aes128_decrypt_cbc(ptx, prev_dec, common_expanded_aes_gcm_key);
            if (memcmp(ptx, ctx, 16))
            {
                printf("Monte Carlo CBC key restore failure\n");
                log_buffer(ctx, 16);
                log_buffer(ptx, 16);
                printf("Error\n");
                exit(2);
            }
            memcpy(prev_ct2, ct, 16);
            memcpy(ct, prev_ct, 16);
            memcpy(prev_ct, prev_ct2, 16);
        }
        printf("CIPHERTEXT =");
        log_buffer(prev_ct, 16);
        printf("\n");
        vec_xor(key, prev_ct, 16);
        memcpy(iv, prev_ct, 16);
        memcpy(pt, ct, 16);
    }
    printf("Monte Carlo CBC test passed\n\n");
}

int main(int argc, char *argv[])
{
#if 1
    long int i;
    long int j;
    static const char *s;
    int mode;
#endif

#if !defined(HOST_BUILD)
    WDTCTL = WDTPW | WDTHOLD;
#endif

    mode = 0;
    monte_carlo = 0;
    if (argc > 1)
    {
        if (strcmp(argv[1], "-e") == 0)
            encrypt = 1;
        else if (strcmp(argv[1], "-d") == 0)
            encrypt = 0;
        else if (strcmp(argv[1], "-m") == 0)
            monte_carlo = 1;
    }

    keylen = 0;
    ivlen = 0;
    ptlen = 0;
    aadlen = 0;
    ctlen = 0;
    taglen = 0;
    fail = 0;

    total_tests = 0;

    if (monte_carlo)
    {
        extract(key, "8d2e60365f17c7df1040d7501b4a7b5a");
        extract(pt, "59b5088e6dadc3ad5f27a460872d5929");
        ecb_monte_carlo_test(key, pt);
        extract(key, "9dc2c84a37850c11699818605f47958c");
        extract(iv, "256953b2feab2a04ae0180d8335bbed6");
        extract(pt, "2e586692e647f5028ec6fa47a55a2aab");
        cbc_monte_carlo_test(key, iv, pt);
        exit(0);
    }

#if !defined(HOST_BUILD)
    encrypt = 0;
    s = gcmDecrypt128;
    for (i = 0;  ;  i++)
    {
        if (s[i] == '\0')
        {
            if (s == gcmEncryptExtIV128)
                break;
            encrypt = 1;
            s = gcmEncryptExtIV128;
            i = 0;
        }
        for (j = 0;  s[i + j];  j++)
        {
            buf[j] = s[i + j];
            if (buf[j] == '\n')
                break;
        }
        buf[j + 1] = '\0';
        i += j;
#define strncasecmp strncmp
#else
    while (fgets(buf, 132, stdin))
    {
#endif
        if (strncasecmp(buf, "Count ", 6) == 0)
        {
            /* Nothing to do */
        }
        else if (strncasecmp(buf, "Key ", 4) == 0)
        {
            keylen = extract(key, buf + 6);
        }
        else if (strncasecmp(buf, "IV ", 3) == 0)
        {
            ivlen = extract(iv, buf + 5);
        }
        else if (strncasecmp(buf, "PT ", 3) == 0)
        {
            ptlen = extract(pt, buf + 5);
        }
        else if (strncasecmp(buf, "AAD ", 4) == 0)
        {
            aadlen = extract(aad, buf + 6);
        }
        else if (strncasecmp(buf, "CT ", 3) == 0)
        {
            ctlen = extract(ct, buf + 5);
        }
        else if (strncasecmp(buf, "Tag ", 4) == 0)
        {
            taglen = extract(tag, buf + 6);
        }
        else if (strncasecmp(buf, "Plaintext ", 10) == 0)
        {
            plaintextlen = extract(plaintext, buf + 12);
        }
        else if (strncasecmp(buf, "Ciphertext ", 11) == 0)
        {
            ciphertextlen = extract(ciphertext, buf + 13);
        }
        else if (strncasecmp(buf, "[", 1) == 0)
        {
            /* Nothing to do */
        }
        else if (strncasecmp(buf, "FAIL", 4) == 0)
        {
            fail = 1;
        }
        else if (buf[0] == ' '  ||  buf[0] == '\n'  ||  buf[0] == '\r')
        {
            if (mode == 0)
                run_ebc_test();
            else
                run_cbc_test();
            keylen = 0;
            ivlen = 0;
            ptlen = 0;
            aadlen = 0;
            ctlen = 0;
            taglen = 0;
            fail = 0;

            total_tests++;
        }
        else
        {
            /* Something unexpected */
        }
    }
    printf("Total tests %d\n", total_tests);
    return 0;
}
